/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#include "convergence/DungeonDoorModel.h"
#include "convergence/PlayerModel.h"

DungeonDoorModel * DungeonDoorModel_create(DungeonRoomStateModel * stateModel, int keyID, struct rect4f bounds) {
	stemobject_create_implementation(DungeonDoorModel, init, stateModel, keyID, bounds);
}

static void sharedInit(DungeonDoorModel * self) {
	self->dispose = DungeonDoorModel_dispose;
	self->copy = DungeonDoorModel_copy;
	self->wouldCollide = DungeonDoorModel_wouldCollide;
	self->roughBounds = DungeonDoorModel_roughBounds;
	self->collisionGeometry = DungeonDoorModel_collisionGeometry;
	self->collision = DungeonDoorModel_collision;
	
	self->type = GAME_OBJECT_TYPE_DOOR;
}

void DungeonDoorModel_init(compat_type(DungeonDoorModel *) selfPtr, DungeonRoomStateModel * stateModel, int keyID, struct rect4f bounds) {
	DungeonDoorModel * self = selfPtr;
	
	GameObjectModel_init(self, stateModel);
	sharedInit(self);
	self->keyID = keyID;
	self->bounds = bounds;
}

void DungeonDoorModel_initCopy(compat_type(GameObjectModel *) selfPtr, compat_type(GameObjectModel *) originalPtr) {
	DungeonDoorModel * self = selfPtr, * original = originalPtr;
	
	GameObjectModel_initCopy(self, original);
	sharedInit(self);
	self->keyID = original->keyID;
	self->bounds = original->bounds;
}

void DungeonDoorModel_dispose(compat_type(DungeonDoorModel *) selfPtr) {
	GameObjectModel_dispose(selfPtr);
}

compat_type(DungeonDoorModel *) DungeonDoorModel_copy(compat_type(DungeonDoorModel *) selfPtr) {
	stemobject_copy_implementation(DungeonDoorModel, initCopy)
}

bool DungeonDoorModel_wouldCollide(compat_type(DungeonDoorModel *) selfPtr, compat_type(GameObjectModel *) objectPtr) {
	DungeonDoorModel * self = selfPtr;
	GameObjectModel * object = objectPtr;
	
	switch (object->type) {
		case GAME_OBJECT_TYPE_ENEMY:
		case GAME_OBJECT_TYPE_PLAYER:
		case GAME_OBJECT_TYPE_PROJECTILE:
			return !self->markedForRemoval;
		default:
			return false;
	}
}

struct rect4f DungeonDoorModel_roughBounds(compat_type(DungeonDoorModel *) selfPtr) {
	DungeonDoorModel * self = selfPtr;
	
	return self->bounds;
}

struct collisionGeometry DungeonDoorModel_collisionGeometry(compat_type(DungeonDoorModel *) selfPtr) {
	DungeonDoorModel * self = selfPtr;
	struct collisionGeometry geometry;
	
	geometry.type = COLLISION_GEOMETRY_TYPE_SWEPT_AABB;
	geometry.u.sweptAABB.current.x = self->bounds.left;
	geometry.u.sweptAABB.current.y = self->bounds.bottom;
	geometry.u.sweptAABB.future = geometry.u.sweptAABB.current;
	geometry.u.sweptAABB.width = self->bounds.right - self->bounds.left;
	geometry.u.sweptAABB.height = self->bounds.top - self->bounds.bottom;
	return geometry;
}

void DungeonDoorModel_collision(compat_type(DungeonDoorModel *) selfPtr, compat_type(GameObjectModel *) objectPtr, const struct collision collision, float time) {
	DungeonDoorModel * self = selfPtr;
	GameObjectModel * object = objectPtr;
	
	if (object->type == GAME_OBJECT_TYPE_PLAYER) {
		PlayerModel * playerModel = objectPtr;
		
		if (playerModel->inventory->hasKey(playerModel->inventory, self->keyID)) {
			self->markedForRemoval = true;
			self->eventDispatcher->dispatchEvent(self->eventDispatcher, Atom_fromString(DUNGEON_DOOR_EVENT_UNLOCKED), NULL);
		}
	}
}
