/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#include "convergence/DungeonKeyModel.h"

DungeonKeyModel * DungeonKeyModel_create(DungeonRoomStateModel * stateModel, int keyID, Vector2 position) {
	stemobject_create_implementation(DungeonKeyModel, init, stateModel, keyID, position)
}

static void sharedInit(DungeonKeyModel * self) {
	self->type = GAME_OBJECT_TYPE_KEY;
	self->dispose = DungeonKeyModel_dispose;
	self->copy = DungeonKeyModel_copy;
	self->roughBounds = DungeonKeyModel_roughBounds;
	self->collisionGeometry = DungeonKeyModel_collisionGeometry;
	self->wouldCollide = DungeonKeyModel_wouldCollide;
	self->collision = DungeonKeyModel_collision;
}

void DungeonKeyModel_init(compat_type(DungeonKeyModel *) selfPtr, DungeonRoomStateModel * stateModel, int keyID, Vector2 position) {
	DungeonKeyModel * self = selfPtr;
	
	GameObjectModel_init(self, stateModel);
	sharedInit(self);
	self->keyID = keyID;
	self->position = position;
}

void DungeonKeyModel_initCopy(compat_type(DungeonKeyModel *) selfPtr, compat_type(DungeonKeyModel *) originalPtr) {
	DungeonKeyModel * self = selfPtr;
	DungeonKeyModel * original = originalPtr;
	
	GameObjectModel_initCopy(self, original);
	sharedInit(self);
	self->keyID = original->keyID;
	self->position = original->position;
}

void DungeonKeyModel_dispose(compat_type(DungeonKeyModel *) selfPtr) {
	DungeonKeyModel * self = selfPtr;
	
	StemObject_dispose(self);
}

compat_type(DungeonKeyModel *) DungeonKeyModel_copy(compat_type(DungeonKeyModel *) selfPtr) {
	stemobject_copy_implementation(DungeonKeyModel, initCopy)
}

struct rect4f DungeonKeyModel_roughBounds(compat_type(DungeonKeyModel *) selfPtr) {
	DungeonKeyModel * self = selfPtr;
	struct rect4f bounds;
	
	bounds.left = self->position.x - KEY_RADIUS * 0.5f;
	bounds.right = self->position.x + KEY_RADIUS * 0.5f;
	bounds.bottom = self->position.y - KEY_RADIUS;
	bounds.top = self->position.y + KEY_RADIUS;
	return bounds;
}

struct collisionGeometry DungeonKeyModel_collisionGeometry(compat_type(DungeonKeyModel *) selfPtr) {
	DungeonKeyModel * self = selfPtr;
	struct collisionGeometry geometry;
	
	geometry.type = COLLISION_GEOMETRY_TYPE_SWEPT_AABB;
	geometry.u.sweptAABB.current.x = self->position.x - KEY_RADIUS * 0.5f;
	geometry.u.sweptAABB.current.y = self->position.y - KEY_RADIUS;
	geometry.u.sweptAABB.future = geometry.u.sweptAABB.current;
	geometry.u.sweptAABB.width = KEY_RADIUS;
	geometry.u.sweptAABB.height = KEY_RADIUS * 2.0f;
	return geometry;
}

bool DungeonKeyModel_wouldCollide(compat_type(DungeonKeyModel *) selfPtr, compat_type(GameObjectModel *) objectPtr) {
	DungeonKeyModel * self = selfPtr;
	GameObjectModel * object = objectPtr;
	
	switch (object->type) {
		case GAME_OBJECT_TYPE_PLAYER:
			return !self->markedForRemoval;
		default:
			return false;
	}
}

void DungeonKeyModel_collision(compat_type(DungeonKeyModel *) selfPtr, compat_type(GameObjectModel *) objectPtr, const struct collision collision, float time) {
	DungeonKeyModel * self = selfPtr;
	
	self->markedForRemoval = true;
}
