/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#ifndef __DUNGEON_MODEL_H__
#define __DUNGEON_MODEL_H__

typedef struct DungeonModel DungeonModel;

#include "3dmath/Vector.h"
#include "convergence/DungeonRoomModel.h"
#include "convergence/SharedDefinitions.h"
#include "serialization/DeserializationContext.h"
#include "serialization/SerializationContext.h"
#include "stemobject/StemObject.h"
#include "utilities/Atom.h"

#define DUNGEON_SERIALIZATION_FORMAT_VERSION 1

struct DungeonModel_exit {
	Atom name;
	Atom room;
	Vector2 position;
};

#define DungeonModel_structContents \
	StemObject_structContents \
	\
	Atom startRoom; \
	Vector2 startPosition; \
	Direction startFacing; \
	size_t roomCount; \
	char ** roomFileNames; \
	size_t exitCount; \
	struct DungeonModel_exit * exits;

struct DungeonModel {
	DungeonModel_structContents
};

DungeonModel * DungeonModel_create();
void DungeonModel_init(compat_type(DungeonModel *) selfPtr);
void DungeonModel_dispose(compat_type(DungeonModel *) selfPtr);

DungeonModel * DungeonModel_deserialize(compat_type(DeserializationContext *) deserializationContext);
bool DungeonModel_loadSerializedData(compat_type(DungeonModel *) selfPtr, compat_type(DeserializationContext *) deserializationContext);
void DungeonModel_serialize(compat_type(DungeonModel *) selfPtr, compat_type(SerializationContext *) serializationContext);

#endif
