/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#ifndef __DUNGEON_ROOM_MODEL_H__
#define __DUNGEON_ROOM_MODEL_H__

typedef struct DungeonRoomModel DungeonRoomModel;

#define DUNGEON_ROOM_SERIALIZATION_FORMAT_VERSION 5

typedef enum DungeonTileType {
	DUNGEON_TILE_TYPE_WALL,
	DUNGEON_TILE_TYPE_FLOOR
} DungeonTileType;

typedef enum DungeonEnemyType {
	DUNGEON_ENEMY_TYPE_BERSERKER,
	DUNGEON_ENEMY_TYPE_RANGER,
	DUNGEON_ENEMY_TYPE_FIRST_BOSS
} DungeonEnemyType;

typedef enum DungeonObjectType {
	DUNGEON_OBJECT_TYPE_HEAL_POWERUP,
	DUNGEON_OBJECT_TYPE_DOOR,
	DUNGEON_OBJECT_TYPE_KEY,
	DUNGEON_OBJECT_TYPE_RANK_UP_POWERUP,
	DUNGEON_OBJECT_TYPE_STAIRCASE
} DungeonObjectType;

#include "3dmath/Vector.h"
#include "convergence/DungeonTileColorMap.h"
#include "convergence/InventoryModel.h"
#include "convergence/SharedDefinitions.h"
#include "serialization/DeserializationContext.h"
#include "serialization/SerializationContext.h"
#include "stemobject/StemObject.h"
#include "utilities/Atom.h"

struct DungeonTile {
	DungeonTileType type;
	Atom canvasTileID;
};

struct DungeonEnemy {
	DungeonEnemyType type;
	int rank;
	Vector2 startPosition;
	Direction startFacing;
	size_t inventoryItemCount;
	struct InventoryModel_item * inventoryItems;
	size_t patrolWaypointCount;
	Vector2 * patrolWaypoints;
};

struct DungeonObject {
	DungeonObjectType type;
	union {
		struct {
			int rank;
			Vector2 position;
		} healPowerup;
		struct {
			int keyID;
			struct rect4f bounds;
		} door;
		struct {
			int keyID;
			Vector2 position;
		} key;
		struct {
			Vector2 position;
		} rankUpPowerup;
		struct {
			Vector2 position;
			bool down;
		} staircase;
	} u;
};

struct DungeonRoomExit {
	Atom name;
	struct rect4f bounds;
};

#define DungeonRoomModel_structContents \
	StemObject_structContents \
	\
	Atom name; \
	char * tileMapFileName; \
	char * colorMapFileName; \
	char * tileCanvasFileName; \
	unsigned int tileCountX; \
	unsigned int tileCountY; \
	struct DungeonTile * tiles; \
	size_t enemyCount; \
	struct DungeonEnemy * enemies; \
	size_t objectCount; \
	struct DungeonObject * objects; \
	size_t exitCount; \
	struct DungeonRoomExit * exits; \
	\
	void (* loadTileMap)(compat_type(DungeonRoomModel *) self, DungeonTileColorMap * colorMap, void * pixels, unsigned int width, unsigned int height, unsigned int bytesPerRow, unsigned int bytesPerPixel);

struct DungeonRoomModel {
	DungeonRoomModel_structContents
};

DungeonRoomModel * DungeonRoomModel_create();
void DungeonRoomModel_init(compat_type(DungeonRoomModel *) selfPtr);
void DungeonRoomModel_dispose(compat_type(DungeonRoomModel *) selfPtr);

DungeonRoomModel * DungeonRoomModel_deserialize(compat_type(DeserializationContext *) deserializationContext);
bool DungeonRoomModel_loadSerializedData(compat_type(DungeonRoomModel *) selfPtr, compat_type(DeserializationContext *) deserializationContext);
void DungeonRoomModel_serialize(compat_type(DungeonRoomModel *) selfPtr, compat_type(SerializationContext *) serializationContext);

// bytesPerPixel must be at least 3
void DungeonRoomModel_loadTileMap(compat_type(DungeonRoomModel *) selfPtr, DungeonTileColorMap * colorMap, void * pixels, unsigned int width, unsigned int height, unsigned int bytesPerRow, unsigned int bytesPerPixel);

#endif
