/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#ifndef __GAME_SESSION_H__
#define __GAME_SESSION_H__

typedef struct GameSession GameSession;

#include "convergence/PlayerInputMap.h"
#include "preferences/Preferences.h"
#include "resourcemanager/ResourceManager.h"
#include "stemobject/StemObject.h"

enum PlayerClass {
	PLAYER_CLASS_BERSERKER,
	PLAYER_CLASS_RANGER,
	PLAYER_CLASS_SPY,
	PLAYER_CLASS_DUELIST,
	PLAYER_CLASS_SPEEDSTER
};

#define GameSession_structContents \
	StemObject_structContents \
	\
	ResourceManager * resourceManager; \
	Preferences * preferences; \
	PlayerInputMap * playerInputMap; \
	enum PlayerClass playerClass; \
	const char * recordInputToFile; \
	const char * playInputFromFile; \
	bool showingInstructionsBeforeGame; \
	\
	void (* saveControlBindings)(compat_type(GameSession *) selfPtr);

struct GameSession {
	GameSession_structContents
};

GameSession * GameSession_create();
void GameSession_init(compat_type(GameSession *) selfPtr);
void GameSession_dispose(compat_type(GameSession *) selfPtr);

void GameSession_saveControlBindings(compat_type(GameSession *) selfPtr);

#endif
