/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#ifndef __GAME_STATE_MODEL_H__
#define __GAME_STATE_MODEL_H__

typedef struct GameStateModel GameStateModel;

#define GAME_STATE_EVENT_PLAYER_DIED "GAME_STATE_EVENT_PLAYER_DIED"
#define GAME_STATE_EVENT_ROOM_CHANGED "GAME_STATE_EVENT_ROOM_CHANGED"
#define GAME_STATE_EVENT_DUNGEON_COMPLETE "GAME_STATE_EVENT_DUNGEON_COMPLETE"

#include "convergence/DungeonModel.h"
#include "convergence/DungeonRoomStateModel.h"
#include "convergence/PlayerModel.h"
#include "resourcemanager/ResourceManager.h"
#include "stemobject/StemObject.h"
#include "utilities/EventDispatcher.h"

#define GameStateModel_structContents \
	StemObject_structContents \
	\
	EventDispatcher * eventDispatcher; \
	DungeonModel * dungeon; \
	size_t roomCount; \
	DungeonRoomStateModel ** rooms; \
	size_t activeRoomIndex; \
	DungeonRoomStateModel * activeRoom; \
	DungeonRoomStateModel * roomCopy; \
	PlayerModel * playerModel; \
	PlayerModel * playerModelCopy; \
	Atom exitName; \
	\
	void (* exit)(compat_type(GameStateModel *) self); \
	void (* restartRoom)(compat_type(GameStateModel *) self);

struct GameStateModel {
	GameStateModel_structContents
};

GameStateModel * GameStateModel_create(DungeonModel * dungeon, enum PlayerClass playerClass, ResourceManager * resourceManager);
void GameStateModel_init(compat_type(GameStateModel *) selfPtr, DungeonModel * dungeon, enum PlayerClass playerClass, ResourceManager * resourceManager);
void GameStateModel_dispose(compat_type(GameStateModel *) selfPtr);

void GameStateModel_exit(compat_type(GameStateModel *) selfPtr);
void GameStateModel_restartRoom(compat_type(GameStateModel *) selfPtr);

#endif
