/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#ifndef __INPUT_CONTROLLER_H__
#define __INPUT_CONTROLLER_H__

typedef struct InputController InputController;

#include "convergence/PlayerInputMap.h"
#include "convergence/GamepadAxisVirtualButton.h"
#include "stemobject/StemObject.h"

#define InputController_structContents \
	StemObject_structContents \
	\
	PlayerInputMap * inputMap; \
	bool leftTriggered; \
	bool rightTriggered; \
	bool downTriggered; \
	bool upTriggered; \
	bool attackTriggered; \
	bool specialTriggered; \
	size_t virtualButtonCount; \
	GamepadAxisVirtualButton ** virtualButtons; \
	int nextDevicePoll; \
	\
	void (* update)(compat_type(InputController *) self); \
	bool (* keyDown)(compat_type(InputController *) self, unsigned int charCode, unsigned int keyCode, unsigned int modifiers); \
	bool (* keyUp)(compat_type(InputController *) self, unsigned int charCode, unsigned int keyCode, unsigned int modifiers); \
	bool (* keyModifiersChanged)(compat_type(InputController *) self, unsigned int modifiers); \
	bool (* protected_ivar(triggerAction))(compat_type(InputController *) self, Atom actionID); \
	bool (* protected_ivar(releaseAction))(compat_type(InputController *) self, Atom actionID);

struct InputController {
	InputController_structContents
};

InputController * InputController_create(PlayerInputMap * inputMap);
void InputController_init(compat_type(InputController *) selfPtr, PlayerInputMap * inputMap);
void InputController_dispose(compat_type(InputController *) selfPtr);

void InputController_update(compat_type(InputController *) selfPtr);
bool InputController_keyDown(compat_type(InputController *) selfPtr, unsigned int charCode, unsigned int keyCode, unsigned int modifiers);
bool InputController_keyUp(compat_type(InputController *) selfPtr, unsigned int charCode, unsigned int keyCode, unsigned int modifiers);
bool InputController_keyModifiersChanged(compat_type(InputController *) selfPtr, unsigned int modifiers);
bool InputController_triggerAction(compat_type(InputController *) selfPtr, Atom actionID);
bool InputController_releaseAction(compat_type(InputController *) selfPtr, Atom actionID);

#endif
