/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#include "convergence/PauseScreenView.h"
#include "convergence/ShellStateGlobals.h"
#include <stddef.h>
#include <string.h>

struct vertex_p2f {
	GLfloat position[2];
};

PauseScreenView * PauseScreenView_create(ResourceManager * resourceManager, int * selectedItemIndex) {
	stemobject_create_implementation(PauseScreenView, init, resourceManager, selectedItemIndex)
}

void PauseScreenView_init(compat_type(PauseScreenView *) selfPtr, ResourceManager * resourceManager, int * selectedItemIndex) {
	PauseScreenView * self = selfPtr;
	struct vertex_p2f vertices[4] = {{{-1.0f, -1.0f}}, {{1.0f, -1.0f}}, {{1.0f, 1.0f}}, {{-1.0f, 1.0f}}};
	GLubyte indexes[4] = {0, 1, 2, 3};
	
	StemObject_init(self);
	self->dispose = PauseScreenView_dispose;
	self->draw = PauseScreenView_draw;
	
	self->resourceManager = resourceManager;
	self->font = self->resourceManager->referenceResource(self->resourceManager, "bitmapfont", "font.json");
	self->selectedItemIndex = selectedItemIndex;
	
	glGenBuffers(1, &self->vboID);
	glGenBuffers(1, &self->eboID);
	glBindBuffer(GL_ARRAY_BUFFER, self->vboID);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, self->eboID);
	glBufferData(GL_ARRAY_BUFFER, sizeof(struct vertex_p2f) * 4, vertices, GL_STATIC_DRAW);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(GLubyte) * 4, indexes, GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
}

void PauseScreenView_dispose(compat_type(PauseScreenView *) selfPtr) {
	PauseScreenView * self = selfPtr;
	
	self->resourceManager->releaseResource(self->resourceManager, "bitmapfont", "font.json");
	glDeleteBuffers(1, &self->vboID);
	glDeleteBuffers(1, &self->eboID);
	StemObject_dispose(self);
}

#define TEXT_HEIGHT 0.0625f

static void drawString(PauseScreenView * self, bool highlighted, const char * string, float y) {
	float stringWidth;
	
	if (highlighted) {
		glColor4f(1.0f, 0.5f, 0.0f, 1.0f);
	} else {
		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	}
	stringWidth = self->font->measureString(self->font, string, strlen(string));
	self->font->drawString(self->font, string, strlen(string), TEXT_HEIGHT, -stringWidth * TEXT_HEIGHT * 0.5f, y - TEXT_HEIGHT * 0.5f, 0.0f);
}

void PauseScreenView_draw(compat_type(PauseScreenView *) selfPtr) {
	PauseScreenView * self = selfPtr;
	float ratio;
	
	ratio = (float) g_windowWidth / (float) g_windowHeight;
	
	glBindBuffer(GL_ARRAY_BUFFER, self->vboID);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, self->eboID);
	glEnable(GL_BLEND);
	glEnableClientState(GL_VERTEX_ARRAY);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glScalef(ratio, 1.0f, 1.0f);
	glVertexPointer(2, GL_FLOAT, sizeof(struct vertex_p2f), (void *) offsetof(struct vertex_p2f, position));
	glColor4f(0.0f, 0.0f, 0.0f, 0.75f);
	glDrawElements(GL_TRIANGLE_FAN, 4, GL_UNSIGNED_BYTE, 0);
	glDisableClientState(GL_VERTEX_ARRAY);
	glDisable(GL_BLEND);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
	
	glLoadIdentity();
	drawString(self, false, "PAUSE", 0.25f);
	drawString(self, *self->selectedItemIndex == 0, "Resume", 0.0f);
	drawString(self, *self->selectedItemIndex == 1, "End game", -0.125f);
}
