/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#ifndef __SHARED_DEFINITIONS_H__
#define __SHARED_DEFINITIONS_H__

#include "3dmath/Vector.h"
#include <math.h>
#include <stdint.h>

#define min(lhs, rhs) ((lhs) < (rhs) ? (lhs) : (rhs))
#define max(lhs, rhs) ((lhs) > (rhs) ? (lhs) : (rhs))

#define stemobject_copy_implementation(class_name, initcopy_suffix, init_args...) \
	class_name * self = selfPtr; \
	class_name * copy; \
	\
	copy = malloc(sizeof(class_name)); \
	class_name##_##initcopy_suffix(copy, self, ##init_args); \
	copy->protected_ivar(allocated) = true; \
	return copy;

#define stemobject_deserialize_implementation(class_name, deserialization_context, load_suffix) \
	DeserializationContext * context = deserialization_context; \
	class_name * self; \
	\
	self = malloc(sizeof(class_name)); \
	if (!class_name##_##load_suffix(self, context)) { \
		free(self); \
		return NULL; \
	} \
	self->protected_ivar(allocated) = true; \
	\
	return self;

struct rect4f {
	float left;
	float right;
	float bottom;
	float top;
};

typedef enum Direction {
	NORTH,
	NORTHEAST,
	EAST,
	SOUTHEAST,
	SOUTH,
	SOUTHWEST,
	WEST,
	NORTHWEST
} Direction;

// Clockwise
static inline float directionToRadians(Direction direction) {
	return fabs(direction * M_PI / 4);
}

static inline Direction intentToDirection(int x, int y) {
	if (x == -1) {
		if (y == -1) {
			return SOUTHWEST;
		}
		if (y == 0) {
			return WEST;
		}
		return NORTHWEST;
	}
	if (x == 0) {
		if (y == -1) {
			return SOUTH;
		}
		return NORTH;
	}
	if (y == -1) {
		return SOUTHEAST;
	}
	if (y == 0) {
		return EAST;
	}
	return NORTHEAST;
}

static inline Vector2 directionToVector(Direction direction) {
	float angle;
	Vector2 result;
	
	angle = directionToRadians(direction);
	result.x = sin(angle);
	result.y = cos(angle);
	return result;
}

static inline Direction vectorToNearestDirection(Vector2 vector) {
	float angle;
	int direction;
	
	angle = atan2(vector.x, vector.y);
	if (angle < 0) {
		angle += M_PI * 2;
	}
	direction = angle * 8.0f / (M_PI * 2) + 0.5f;
	return ((Direction) min(max(direction, NORTH), NORTHWEST + 1)) % 8;
}

#endif
