/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#include "unittest/framework/TestSuite.h"
#include "convergence/AIMath.h"

static void testObjectsInLineOfSight() {
}

static void testBestPathToPosition() {
	struct DungeonTile * tiles;
	struct travelPath path;
	
	tiles = malloc(sizeof(struct DungeonTile) * 3 * 3);
	tiles[0].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[1].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[2].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[3].type = DUNGEON_TILE_TYPE_WALL;
	tiles[4].type = DUNGEON_TILE_TYPE_WALL;
	tiles[5].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[6].type = DUNGEON_TILE_TYPE_WALL;
	tiles[7].type = DUNGEON_TILE_TYPE_WALL;
	tiles[8].type = DUNGEON_TILE_TYPE_FLOOR;
	
	path = AIBestPathToDestination(0, 8, tiles, 3, 3);
	TestCase_assert(path.nodeCount == 4, "Expected 4 but got %zu", path.nodeCount);
	TestCase_assert(path.nodes != NULL, "Expected non-NULL but got NULL");
	TestCase_assert(path.nodes[0] == 1, "Expected 1 but got %d", path.nodes[0]);
	TestCase_assert(path.nodes[1] == 2, "Expected 2 but got %d", path.nodes[1]);
	TestCase_assert(path.nodes[2] == 5, "Expected 5 but got %d", path.nodes[2]);
	TestCase_assert(path.nodes[3] == 8, "Expected 8 but got %d", path.nodes[3]);
	free(path.nodes);
	
	tiles[0].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[1].type = DUNGEON_TILE_TYPE_WALL;
	tiles[2].type = DUNGEON_TILE_TYPE_WALL;
	tiles[3].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[4].type = DUNGEON_TILE_TYPE_WALL;
	tiles[5].type = DUNGEON_TILE_TYPE_WALL;
	tiles[6].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[7].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[8].type = DUNGEON_TILE_TYPE_FLOOR;
	
	path = AIBestPathToDestination(0, 8, tiles, 3, 3);
	TestCase_assert(path.nodeCount == 4, "Expected 4 but got %zu", path.nodeCount);
	TestCase_assert(path.nodes != NULL, "Expected non-NULL but got NULL");
	TestCase_assert(path.nodes[0] == 3, "Expected 3 but got %d", path.nodes[0]);
	TestCase_assert(path.nodes[1] == 6, "Expected 6 but got %d", path.nodes[1]);
	TestCase_assert(path.nodes[2] == 7, "Expected 7 but got %d", path.nodes[2]);
	TestCase_assert(path.nodes[3] == 8, "Expected 8 but got %d", path.nodes[3]);
	free(path.nodes);
	
	tiles[0].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[1].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[2].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[3].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[4].type = DUNGEON_TILE_TYPE_WALL;
	tiles[5].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[6].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[7].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[8].type = DUNGEON_TILE_TYPE_FLOOR;
	
	path = AIBestPathToDestination(1, 7, tiles, 3, 3);
	TestCase_assert(path.nodeCount == 4, "Expected 4 but got %zu", path.nodeCount);
	TestCase_assert(path.nodes != NULL, "Expected non-NULL but got NULL");
	TestCase_assert(path.nodes[0] == 0 || path.nodes[0] == 2, "Expected 0 or 2 but got %d", path.nodes[0]);
	TestCase_assert(path.nodes[1] == path.nodes[0] + 3, "Expected %d but got %d", path.nodes[0] + 3, path.nodes[1]);
	TestCase_assert(path.nodes[2] == path.nodes[0] + 6, "Expected %d but got %d", path.nodes[0] + 6, path.nodes[2]);
	TestCase_assert(path.nodes[3] == 7, "Expected 7 but got %d", path.nodes[3]);
	free(path.nodes);
	
	path = AIBestPathToDestination(0, 8, tiles, 3, 3);
	TestCase_assert(path.nodeCount == 4, "Expected 4 but got %zu", path.nodeCount);
	TestCase_assert(path.nodes != NULL, "Expected non-NULL but got NULL");
	TestCase_assert(path.nodes[0] == 1 || path.nodes[0] == 3, "Expected 1 or 3 but got %d", path.nodes[0]);
	if (path.nodes[0] == 1) {
		TestCase_assert(path.nodes[1] == 2, "Expected 2 but got %d", path.nodes[1]);
		TestCase_assert(path.nodes[2] == 5, "Expected 5 but got %d", path.nodes[2]);
	} else {
		TestCase_assert(path.nodes[1] == 6, "Expected 6 but got %d", path.nodes[1]);
		TestCase_assert(path.nodes[2] == 7, "Expected 7 but got %d", path.nodes[2]);
	}
	TestCase_assert(path.nodes[3] == 8, "Expected 8 but got %d", path.nodes[3]);
	free(path.nodes);
	
	tiles[0].type = DUNGEON_TILE_TYPE_WALL;
	tiles[1].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[2].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[3].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[4].type = DUNGEON_TILE_TYPE_WALL;
	tiles[5].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[6].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[7].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[8].type = DUNGEON_TILE_TYPE_FLOOR;
	
	path = AIBestPathToDestination(1, 3, tiles, 3, 3);
	TestCase_assert(path.nodeCount == 6, "Expected 6 but got %zu", path.nodeCount);
	TestCase_assert(path.nodes != NULL, "Expected non-NULL but got NULL");
	TestCase_assert(path.nodes[0] == 2, "Expected 2 but got %d", path.nodes[0]);
	TestCase_assert(path.nodes[1] == 5, "Expected 5 but got %d", path.nodes[1]);
	TestCase_assert(path.nodes[2] == 8, "Expected 8 but got %d", path.nodes[2]);
	TestCase_assert(path.nodes[3] == 7, "Expected 7 but got %d", path.nodes[3]);
	TestCase_assert(path.nodes[4] == 6, "Expected 6 but got %d", path.nodes[4]);
	TestCase_assert(path.nodes[5] == 3, "Expected 3 but got %d", path.nodes[5]);
	free(path.nodes);
	
	tiles[0].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[1].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[2].type = DUNGEON_TILE_TYPE_WALL;
	tiles[3].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[4].type = DUNGEON_TILE_TYPE_WALL;
	tiles[5].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[6].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[7].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[8].type = DUNGEON_TILE_TYPE_FLOOR;
	
	path = AIBestPathToDestination(1, 5, tiles, 3, 3);
	TestCase_assert(path.nodeCount == 6, "Expected 6 but got %zu", path.nodeCount);
	TestCase_assert(path.nodes != NULL, "Expected non-NULL but got NULL");
	TestCase_assert(path.nodes[0] == 0, "Expected 0 but got %d", path.nodes[0]);
	TestCase_assert(path.nodes[1] == 3, "Expected 3 but got %d", path.nodes[1]);
	TestCase_assert(path.nodes[2] == 6, "Expected 6 but got %d", path.nodes[2]);
	TestCase_assert(path.nodes[3] == 7, "Expected 7 but got %d", path.nodes[3]);
	TestCase_assert(path.nodes[4] == 8, "Expected 8 but got %d", path.nodes[4]);
	TestCase_assert(path.nodes[5] == 5, "Expected 5 but got %d", path.nodes[5]);
	free(path.nodes);
	
	tiles[0].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[1].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[2].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[3].type = DUNGEON_TILE_TYPE_WALL;
	tiles[4].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[5].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[6].type = DUNGEON_TILE_TYPE_WALL;
	tiles[7].type = DUNGEON_TILE_TYPE_WALL;
	tiles[8].type = DUNGEON_TILE_TYPE_FLOOR;
	
	path = AIBestPathToDestination(0, 8, tiles, 3, 3);
	TestCase_assert(path.nodeCount == 3, "Expected 3 but got %zu", path.nodeCount);
	TestCase_assert(path.nodes != NULL, "Expected non-NULL but got NULL");
	TestCase_assert(path.nodes[0] == 1, "Expected 1 but got %d", path.nodes[0]);
	TestCase_assert(path.nodes[1] == 5, "Expected 5 but got %d", path.nodes[1]);
	TestCase_assert(path.nodes[2] == 8, "Expected 8 but got %d", path.nodes[2]);
	free(path.nodes);
	
	tiles[0].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[1].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[2].type = DUNGEON_TILE_TYPE_WALL;
	tiles[3].type = DUNGEON_TILE_TYPE_WALL;
	tiles[4].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[5].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[6].type = DUNGEON_TILE_TYPE_WALL;
	tiles[7].type = DUNGEON_TILE_TYPE_WALL;
	tiles[8].type = DUNGEON_TILE_TYPE_FLOOR;
	
	path = AIBestPathToDestination(0, 8, tiles, 3, 3);
	TestCase_assert(path.nodeCount == 4, "Expected 4 but got %zu", path.nodeCount);
	TestCase_assert(path.nodes != NULL, "Expected non-NULL but got NULL");
	TestCase_assert(path.nodes[0] == 1, "Expected 1 but got %d", path.nodes[0]);
	TestCase_assert(path.nodes[1] == 4, "Expected 4 but got %d", path.nodes[1]);
	TestCase_assert(path.nodes[2] == 5, "Expected 5 but got %d", path.nodes[2]);
	TestCase_assert(path.nodes[3] == 8, "Expected 8 but got %d", path.nodes[3]);
	free(path.nodes);
	
	tiles[0].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[1].type = DUNGEON_TILE_TYPE_WALL;
	tiles[2].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[3].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[4].type = DUNGEON_TILE_TYPE_WALL;
	tiles[5].type = DUNGEON_TILE_TYPE_WALL;
	tiles[6].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[7].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[8].type = DUNGEON_TILE_TYPE_FLOOR;
	
	path = AIBestPathToDestination(0, 2, tiles, 3, 3);
	TestCase_assert(path.nodeCount == 0, "Expected 0 but got %zu", path.nodeCount);
	TestCase_assert(path.nodes == NULL, "Expected NULL but got %p", path.nodes);
	
	tiles[0].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[1].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[2].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[3].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[4].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[5].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[6].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[7].type = DUNGEON_TILE_TYPE_FLOOR;
	tiles[8].type = DUNGEON_TILE_TYPE_FLOOR;
	
	path = AIBestPathToDestination(5, 8, tiles, 3, 3);
	TestCase_assert(path.nodeCount == 1, "Expected 1 but got %zu", path.nodeCount);
	TestCase_assert(path.nodes != NULL, "Expected non-NULL but got NULL");
	TestCase_assert(path.nodes[0] == 8, "Expected 8 but got %d", path.nodes[0]);
	free(path.nodes);
	
	free(tiles);
}

TEST_SUITE(AIMathTest,
           testObjectsInLineOfSight,
           testBestPathToPosition)
